@@ -22,6 +22,7 @@ bin=$pwd
 helper=$pwd/env_check
 setuid_helper=${tmpdir}/env_check
 helper_sh=$pwd/env_check.sh
+read_locale="/usr/lib/locale/**:r"
 
 # TEST environment filtering on elf binaries
 genprofile $helper:ux
@@ -61,19 +62,19 @@ genprofile ${helper_sh}:Ux
 runchecktest "ENVIRON (shell script): Ux & regular env" pass ${helper_sh} FOO=BAR
 runchecktest "ENVIRON (shell script): Ux & sensitive env" fail ${helper_sh} LD_LIBRARY_PATH=.
 
-genprofile ${helper_sh}:px -- image=${helper_sh}
+genprofile ${helper_sh}:px -- image=${helper_sh} "$read_locale"
 runchecktest "ENVIRON (shell script): px & regular env" pass ${helper_sh} FOO=BAR
 runchecktest "ENVIRON (shell script): px & sensitive env" pass ${helper_sh} LD_LIBRARY_PATH=.
 
-genprofile ${helper_sh}:Px -- image=${helper_sh}
+genprofile ${helper_sh}:Px -- image=${helper_sh} "$read_locale"
 runchecktest "ENVIRON (shell script): Px & regular env" pass ${helper_sh} FOO=BAR
 runchecktest "ENVIRON (shell script): Px & sensitive env" fail ${helper_sh} LD_LIBRARY_PATH=.
 
-genprofile addimage:${helper_sh}
+genprofile addimage:${helper_sh} "$read_locale"
 runchecktest "ENVIRON (shell script): ix & regular env" pass ${helper_sh} FOO=BAR
 runchecktest "ENVIRON (shell script): ix & sensitive env" pass ${helper_sh} LD_LIBRARY_PATH=.
 
-genprofile image=${helper_sh}
+genprofile image=${helper_sh} "$read_locale"
 runchecktest "ENVIRON (shell script): unconfined --> confined & regular env" pass ${helper_sh} FOO=BAR
 runchecktest "ENVIRON (shell script): unconfined --> confined & sensitive env" pass ${helper_sh} LD_LIBRARY_PATH=.
 
